/*Initializes a new incomming TCP/IP connection and handles the REPLAY request (receive 100 bytes and send them back to the client*/
#include <stdio.h>
#include <stdlib.h>
#include "tvsserver_base.h"
#include "tvs_redefine_api_names.h"
#include "c_example_init.h"
#include "c_example_handle.h"

int __stdcall _HandleConnection(ProtocolSession *psess, ClientSession *sess) {
    char input [100];
    int len;
    ProtocolData* pdata = FindProtocolDataStruct(psess, sess->data);
	MyReplaystruct* replayData = (MyReplaystruct*) &pdata->data;
    MyTCPIPstruct* tcpData = (MyTCPIPstruct*) &pdata->data;
 
    if(psess==tcpip_psess) {
        //call the HandleConnection function of the other binded protocols
        CallNextHandleConnection(psess, sess);
        //Close the socket, since we don't need it annymore
		_CloseClientSession(psess, sess);
        return 1;
    } else if(psess==replay_psess) {
        len=100;
        //receive 100 bytes (or less) and wait for a maximum time of 10 seconds
        RecvData(psess, sess, RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT, 10*1000, &input, &len);
        //Send the data that the client just send to us received.
        SendData(psess, sess, 0, 5*1000, &input, &len);

		//Send server path to the client
		len=strlen(replayData->serverPath);
		SendData(psess, sess, 0, 5*1000, replayData->serverPath, &len);
        return 1; //we are done, return so the other protocols can close the connection
    }
    return 0;
}

/*In this example we ignore the timeout parameter and all flags (except for RSF_PEEK_DATA) to keep the example simple*/
int __stdcall _RecvData(ProtocolSession *psess, ClientSession *sess, int flags, int timeout, char *buf, int *len) {
    ProtocolData* pdata = FindProtocolDataStruct(psess, sess->data);
    MyTCPIPstruct* tcpData = (MyTCPIPstruct*) &pdata->data;

	*len = recv(tcpData->socket, buf, *len, flags&RSF_PEEK_DATA); //RSF_PEEK_DATA == MSG_PEEK

	return CE_NOERR;
}

/*In this example we ignore the timeout parameter and all flags to keep the example simple*/
int __stdcall _SendData(ProtocolSession *psess, ClientSession *sess, int flags, int timeout, char *buf, int *len) {
    ProtocolData* pdata = FindProtocolDataStruct(psess, sess->data);
    MyTCPIPstruct* tcpData = (MyTCPIPstruct*) &pdata->data;

	*len = send(tcpData->socket, buf, *len, 0);

	return CE_NOERR;
}

//Close the socket
int __stdcall _CloseClientSession(ProtocolSession *psess, ClientSession *sess){
    ProtocolData* pdata = FindProtocolDataStruct(psess, sess->data);
    MyTCPIPstruct* tcpData = (MyTCPIPstruct*) &pdata->data;
	char myBuf[80];
	unsigned int retv=1;

	//Send message that the connection will be closed
	shutdown(tcpData->socket,SD_SEND);

	//Receive all data that is left in the buffer
	while(retv!=0 && retv+1!=0)
		retv = recv(tcpData->socket, (char*) &myBuf, 80, 0);

	//Close the socket
	closesocket(tcpData->socket);

	return 1;
}