#include "tvsserver_base.h"
#include "c_example_init.h"
#include <stdio.h>
#include <stdlib.h>
#include "tvs_redefine_api_names.h"

char* supportedProtocols = "REPLAY\0C_TCPIP\0\0";
ProtocolSession *replay_psess;
ProtocolSession *tcpip_psess;

/*This function should return a list of protocols it supports*/
int APIENTRY _GetProtocolNames(
	__out	char *buf,
	__inout	int *len
) {
    int size;
 
    if(len==0) return 0;

    size = GetNullTerminatedListSize(supportedProtocols); //calculate the size of the list

    //if the buffer is not big enough, return the size that is needed
    if(*len==0 || *len<size) {
        *len = size;
            return 1;
    }
 
    //copy the list to the buffer and return
	memcpy(buf, supportedProtocols, size);
    return 1;
}
 
/*This function will calculate the size, in bytes, of a null-terminated list*/
int GetNullTerminatedListSize(char* list) {
    int cnt=0;
    int len;
 
    if(list==0) return 0;
     
    while(*list!=0) { //The terminating null byte is found, this is the end of the list
        len = strlen(list);
        list += len + 1; //add the size of the item +1 to get the next item
        cnt += len + 1;
    }
    cnt++; //add the last zero byte
 
    return cnt; //return the size of the list
}

int APIENTRY _StartProtocol(ProtocolSession *psess) {
    ControlPanelOptions *cpo;
    WORD wVersionRequested;
    WSADATA wsaData;
    int retv=0;
    char* tempProtocolSignatureStructure=0;
 
    /* Use the MAKEWORD(lowbyte, highbyte) macro declared in Windef.h */
    wVersionRequested = MAKEWORD(2, 2);
 
    if(!strcmp(psess->protocolName, "REPLAY")) { //init the REPLAY protocol
        replay_psess = psess;

 
        //Set the protocol type to PT_LABORER, make room for the MyReplaystruct structure in the ListenSession and ClientSession and pass the supported API version.
		if(!SetProtocolInformation(
            psess, 
            PT_LABORER,
            (ProtocolSignature*) 0,
            sizeof(MyReplaystruct),
            V_TV_SERVER_1)
        )
            return false;
 
        //Create a ControlPanelOptions strcuture with room for 1 suboption.
        cpo = (ControlPanelOptions*) malloc(
            sizeof(ControlPanelOptions) + 
            sizeof(ControlPanelOption)*REPLAY_AMMOUNT_OF_SUB_OPTIONS // +1 subOption
        );
 
        /*Empty memory and fill in the size, in bytes*/
        memset(cpo, 0, sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*REPLAY_AMMOUNT_OF_SUB_OPTIONS);
        cpo->size = sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*REPLAY_AMMOUNT_OF_SUB_OPTIONS;
 
        //The name of this protocol, communicated to the user, will be "REPLAY-C-plugin"
        cpo->main.size = sizeof(ControlPanelOption);
        cpo->main.optionName  = "REPLAY-C-plugin"; //the name of this protocol
 
        //suboption number 1
        cpo->subOptions[0].size = sizeof(ControlPanelOption);
        cpo->subOptions[0].name = "opt1"; //identifier name used by the function GetPluginData and SavePluginData
        cpo->subOptions[0].optionName = "Server-path"; //label of this option
        cpo->subOptions[0].userValue = (UserValueOption*) malloc(sizeof(UserValueOption));
        cpo->subOptions[0].userValue->type = TD_ASCII_STRING; //the pointer on defaultValue is a ascii string.
        cpo->subOptions[0].userValue->defaultValue = "c:\\WWWroot\\"; //default value of this option
        cpo->subOptions[0].userValue->defaultValueSize = strlen((char*) cpo->subOptions[0].userValue->defaultValue); 
        cpo->subOptions[0].flags = CPO_GEN_LISTENSESSION_OPTION; //a valid option for the listensession

        SetControlPanelOptions(psess, cpo);

        retv = AcceptProtocol(psess); //retv=false the protocol is rejected, retv=true the protocol is loaded
 
        free(cpo->subOptions[0].userValue);
        free(cpo);
 
    } else if(!strcmp(psess->protocolName, "C_TCPIP")) { //init the TCP/IP protocol
        tcpip_psess = psess;

        if(!SetProtocolInformation(
            psess, //ProtocolSession
            PT_RECEIVER, //Set the type to receiver protocol
            0, //No Protcol signature for for a PT_RECEIVER protocol
            sizeof(MyTCPIPstruct), //In every ListenSession and ClientSession I want enough room in the data member for my structure
            V_TV_SERVER_1) //This protocol supports API version 1
        )
            return false;
 
        //Create a ControlPanelOptions strcuture with room for 1 suboption.
        cpo = (ControlPanelOptions*) malloc(
            sizeof(ControlPanelOptions) + 
            sizeof(ControlPanelOption)*TCPIP_AMMOUNT_OF_SUB_OPTIONS // +2 subOption
        );
 
        /*Empty memory and fill in the size, in bytes*/
        memset(cpo, 0, sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*TCPIP_AMMOUNT_OF_SUB_OPTIONS);
        cpo->size = sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*TCPIP_AMMOUNT_OF_SUB_OPTIONS;
 
        //The name of this protocol, communicated to the user, will be "TCP/IP-C-plugin"
        cpo->main.size = sizeof(ControlPanelOption);
        cpo->main.optionName  = "TCP/IP-C-plugin"; //the name of this protocol
 
        //suboption number 1
        cpo->subOptions[0].size = sizeof(ControlPanelOption);
        cpo->subOptions[0].name = (char*) OPT_TCP_PORT; //identifier integer, instead of a name used by the function GetPluginData and SavePluginData
        cpo->subOptions[0].optionName = "Port"; //label of this option
        cpo->subOptions[0].userValue = (UserValueOption*) malloc(sizeof(UserValueOption));
        cpo->subOptions[0].userValue->type = TD_SHORT_INT; //the defaultValue is an integer
        cpo->subOptions[0].userValue->defaultValue = (void*) 80; //default value of this option
        cpo->subOptions[0].userValue->defaultValueSize = 0; 
        cpo->subOptions[0].flags = CPO_GEN_LISTENSESSION_OPTION | GDPD_NAME_IS_INTEGER | CPO_GEN_LISTENSESSION_READONLY; //a valid option for the listensession, member name is an integer and cannot be modified on a running service
 
        //suboption number 2
        cpo->subOptions[1].size = sizeof(ControlPanelOption);
        cpo->subOptions[1].name = (char*) OPT_TCP_VERSION6; //identifier integer, instead of a name used by the function GetPluginData and SavePluginData
        cpo->subOptions[1].optionName = "Use version6"; //label of this option
        cpo->subOptions[1].userValue = (UserValueOption*) malloc(sizeof(UserValueOption));
        cpo->subOptions[1].userValue->type = TD_BOOLEAN; //the defaultValue is an boolean
        cpo->subOptions[1].userValue->defaultValue = (void*) 0; //default value of this option is false
        cpo->subOptions[1].userValue->defaultValueSize = 0; 
        cpo->subOptions[1].flags = CPO_GEN_LISTENSESSION_OPTION | GDPD_NAME_IS_INTEGER | CPO_GEN_LISTENSESSION_READONLY; //a valid option for the listensession, member name is an integer and cannot be modified on a running service

        SetControlPanelOptions(psess, cpo);
 
        //Startup the Socket stack
        if(WSAStartup(wVersionRequested, &wsaData))
            return 0;
 
        //We will not make a call to SetControlPanelOptions and immediately call AcceptProtocol
        retv = AcceptProtocol(psess); //retv=false the protocol is rejected, retv=true the protocol is loaded
 
        free(cpo->subOptions[0].userValue);
        free(cpo);
    }
 
    //Free the ProtocolSignatureStructure if allocated
    if(tempProtocolSignatureStructure)
        free(tempProtocolSignatureStructure); 
 
    return retv;
}